/*
 Logic Observer - a CPLD Based Logic Analyser
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/12/2013
 Last Modified: 28/02/2015
 
 */

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Setting elements
#define TRIGGER_PINS 0
#define TRIGGER_LEVEL 1
#define TRIGGER_DELAY 2
#define TRIGGER_COUNT 3
#define SAMPLE_RATE 4
#define SAMPLES 5
#define CAPTURE 6

uint32_t samples_size[7] = {8128, 16002, 32004, 64008, 96012, 131064};

void setup(void) {
	// Set outputs
	DDRD |= ((1<<cpldresetPin) | (1<<startcapturePin) | (1<<analogswitchPin));
	DDRB |= ((1<<nextaddressPin) | (1<<readingPin) | (1<<captureLED));
	
	// Setup Timer 0 at 10MHz
	DDRD |= (1<<PD6); // OCR0A as output
	TCCR0A |= (1<<COM0A0) | (1<<WGM01); // Toggle OC0A on Compare Match
	OCR0A = 0;
	TCCR0B |= (1<<CS00); // No prescaler
	
	// Use AVR Timer to CPLD at start up
	PORTD |= (1<<analogswitchPin);
	
	// Setup Timer 1 overflow for usbPoll
	TIMSK1 |= (1<<TOIE1);
	
	// Setup Timer 2 overflow for trigger delay
	TIMSK2 |= (1<<TOIE2);
	
	sei(); // Turn on interrupts
}

// Begin V-USB default required code
// Start USB communications
void startUSB(void) {
	usbDeviceDisconnect();
	_delay_ms(300); // 300 ms disconnect
	usbDeviceConnect();
	usbInit();
}
// End V-USB default required code
