module Logic_Observer_v1_0 (clockin, reset, reading, address, startcapture, nextaddress, ce, we, oe);

input clockin;
input reset;
input reading;

output [16:0] address;
reg [16:0] address;
reg [16:0] startaddress;

input startcapture;
reg startcaptureinit;
reg capturedone;

input nextaddress;
reg nextaddresswait;
reg readinginit;

output ce;
wire ce;
output we;
reg we;
output oe;
reg oe;

always @ (posedge clockin) begin
	if (!reset) begin
		address <= 17'b0;
		nextaddresswait <= 1'b0;
		readinginit <= 1'b1;
		startcaptureinit <= 1'b1;
		capturedone <= 1'b0;
		we <= 1'b1;
		oe <= 1'b1;
	end
	else begin
		// Start writing and wait for a trigger as long as the capture isn't finished
		if (!reading && !capturedone) begin
			we <= 1'b0;
			oe <= 1'b1;
			readinginit <= 1'b1;
			address <= address + 17'b1;
			
			// Triggered, save the starting address
			if (startcapture && startcaptureinit) begin
				startaddress <= address - 17'd290; // Also record what happened before the trigger (50 clocks before)
				startcaptureinit <= 1'b0;
			end
			
			// Current address is now the start address, stop the capture
			if (startcapture && (startaddress == address)) begin
			  capturedone <= 1'b1;
			end
		end
		
		// Start reading
		else if (reading) begin
			oe <= 1'b0;
			we <= 1'b1;
		  
			// Initialise the starting read address as the start of the capture
			if (readinginit == 1) begin
				readinginit <= 1'b0;
				address <= startaddress;
			end
		  
			// Increment address by 1 when we receive a low and then high clock
			else if (nextaddress == 1 && nextaddresswait == 0) begin
				nextaddresswait <= 1'b1;
				address <= address + 17'b1;
			end
			else if (nextaddress == 0) begin
				nextaddresswait <= 1'b0;
			end
		end
	end
end

assign ce = (reset) ? clockin : 1'b1;

endmodule
